#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author: Filippo Rivato
# Date:	11 March 2016
# Done in Annonay at Hotel "Du midi" after a business trip and 10 hours of work.

from __future__ import absolute_import
from __future__ import print_function
__author__ = "Filippo Rivato"
__email__  = "f.rivato@gmail.com"

__name__ = _("Hilbert")
__version__= "0.0.1"

from CNC import CNC,Block
from ToolsPage import Plugin


#==============================================================================
#Hilbert class
#==============================================================================
class Hilbert:
	def __init__(self,name="Hilbert"):
		self.name = name

	#----------------------------------------------------------------------
	def hilbert(self,x0, y0, xi, xj, yi, yj, n):
		def x():
			return (x0 + (xi + yi)/2.)
		def y():
			return (y0 + (xj + yj)/2.)

		if n >0:
			for ye in self.hilbert(x0,               y0,               yi/2, yj/2, xi/2, xj/2, n - 1):
				yield ye
			for ye in self.hilbert(x0 + xi/2,        y0 + xj/2,        xi/2, xj/2, yi/2, yj/2, n - 1):
				yield ye
			for ye in self.hilbert(x0 + xi/2 + yi/2, y0 + xj/2 + yj/2, xi/2, xj/2, yi/2, yj/2, n - 1):
				yield ye
			for ye in self.hilbert(x0 + xi/2 + yi,   y0 + xj/2 + yj,  -yi/2,-yj/2,-xi/2,-xj/2, n - 1):
				yield ye
		else:
			yield (x(),y())

	#----------------------------------------------------------------------
	def make(self,n = 2, size = 100, depth = 0):
		self.n = n
		self.size = size
		self.depth = depth

		blocks = []
		block = Block(self.name)

		xi,yi = zip(*(self.hilbert(0.0,0.0,size,0.0,0.0,size,n)))

		block.append(CNC.zsafe())
		block.append(CNC.grapid(xi[0],yi[0]))

		currDepth = 0.
		stepz = CNC.vars['stepz']
		if stepz==0 : stepz=0.001  #avoid infinite while loop

		while True:
			currDepth -= stepz
			if currDepth < self.depth : currDepth = self.depth
			block.append(CNC.zenter(currDepth))
			block.append(CNC.gcode(1, [("f",CNC.vars["cutfeed"])]))
			for x,y in zip(xi,yi):
				block.append(CNC.gline(x,y))
			if currDepth <= self.depth : break

		block.append(CNC.zsafe())
		blocks.append(block)
		return blocks


#==============================================================================
# Create a Hilbert curve
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Create a Hilbert path")

	def __init__(self, master):
		Plugin.__init__(self, master, "Hilbert")
		self.icon  = "hilbert"
		self.group = "Artistic"
		self.variables = [
			("name",     "db" ,      "", _("Name")),
			("Size"  ,   "mm" ,    50.0, _("Size")),
			("Order"  , "int" ,       2, _("Order")),
			("Depth"  , "int" ,       0, _("Depth"))
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		name = self["name"]
		if not name or name=="default": name="Hilbert"
		H = Hilbert(name)

		size = self.fromMm("Size")
		n = self["Order"]
		depth = self["Depth"]

		#Check parameters
		if size <=0:
			app.setStatus(_("Hilbert abort: verify the size"))
			return

		if depth >0:
			app.setStatus(_("Hilbert abort: depth must be minor or equal to zero"))
			return

		blocks = H.make(n,size,depth)

		active = app.activeBlock()
		if active==0: active=1
		app.gcode.insBlocks(active, blocks, "Hilbert")
		app.refresh()
		app.setStatus(_("Generated: Hilbert"))

if __name__=="__main__":
	Hilbert = Hilbert()
	Hilbert.make()



from __future__ import absolute_import
import sys
import random
import argparse

from . import stl


def _get_parser(description):
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('infile', nargs='?', type=argparse.FileType('rb'),
                        default=sys.stdin, help='STL file to read')
    parser.add_argument('outfile', nargs='?', type=argparse.FileType('wb'),
                        default=sys.stdout, help='STL file to write')
    parser.add_argument('--name', nargs='?', help='Name of the mesh')
    parser.add_argument(
        '-n', '--use-file-normals', action='store_true',
        help='Read the normals from the file instead of recalculating them')
    parser.add_argument(
        '-r', '--remove-empty-areas', action='store_true',
        help='Remove areas with 0 surface areas to prevent errors during '
        'normal calculation')
    parser.add_argument('-s', '--disable-speedups', action='store_true',
                        help='Disable Cython speedups')
    return parser


def _get_name(args):
    names = [
        args.name,
        getattr(args.outfile, 'name', None),
        getattr(args.infile, 'name', None),
        'numpy-stl-%06d' % random.randint(0, 1e6),
    ]

    for name in names:  # pragma: no branch
        if name and isinstance(name, str) and not name.startswith('<'):
            return name


def main():
    parser = _get_parser('Convert STL files from ascii to binary and back')
    parser.add_argument('-a', '--ascii', action='store_true',
                        help='Write ASCII file (default is binary)')
    parser.add_argument('-b', '--binary', action='store_true',
                        help='Force binary file (for TTYs)')

    args = parser.parse_args()
    name = _get_name(args)
    stl_file = stl.StlMesh(filename=name,
                           fh=args.infile,
                           calculate_normals=False,
                           remove_empty_areas=args.remove_empty_areas,
                           speedups=not args.disable_speedups)

    if args.binary:
        mode = stl.BINARY
    elif args.ascii:
        mode = stl.ASCII
    else:
        mode = stl.AUTOMATIC

    stl_file.save(name, args.outfile, mode=mode,
                  update_normals=not args.use_file_normals)


def to_ascii():
    parser = _get_parser('Convert STL files to ASCII (text) format')
    args = parser.parse_args()
    name = _get_name(args)
    stl_file = stl.StlMesh(filename=name, fh=args.infile,
                           calculate_normals=False,
                           remove_empty_areas=args.remove_empty_areas,
                           speedups=not args.disable_speedups)
    stl_file.save(name, args.outfile, mode=stl.ASCII,
                  update_normals=not args.use_file_normals)


def to_binary():
    parser = _get_parser('Convert STL files to ASCII (text) format')
    args = parser.parse_args()
    name = _get_name(args)
    stl_file = stl.StlMesh(filename=name, fh=args.infile,
                           calculate_normals=False,
                           remove_empty_areas=args.remove_empty_areas,
                           speedups=not args.disable_speedups)
    stl_file.save(name, args.outfile, mode=stl.BINARY,
                  update_normals=not args.use_file_normals)



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author: Filippo Rivato
# Date: 9 November 2015
# A special thanks to Vasilis for his patient explanations

from __future__ import absolute_import
from __future__ import print_function

__author__ = "Filippo Rivato"
__email__  = "f.rivato@gmail.com"

__name__ = _("Driller")
__version__= "0.0.10"

#import os.path
#import re
import math
from collections import OrderedDict
from CNC import CNC, Block
from ToolsPage import Plugin
import Utils

#==============================================================================
#Driller class
#==============================================================================
class Driller:
	def __init__(self,name="Driller"):
		self.name = name


#==============================================================================
# Create holes along selected blocks
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Create holes along selected blocks")

	def __init__(self, master):
		Plugin.__init__(self, master, "Driller")
		self.icon  = "driller"
		self.group = "CAM"

		self.variables = [
			("name",          "db",    "",    _("Name")),
			("HolesDistance", "mm",    10.0,  _("Distance between holes")),
			("TargetDepth",   "mm",    0.0,   _("Target Depth")),
			("Peck",          "mm",    0.0,   _("Peck, 0 means None")),
			("Dwell",         "float", 0.0,   _("Dwell time, 0 means None")),
			("useAnchor",     "bool",  False, _("Use anchor")),
			("File",          "file",  "",    _("Excellon-File")),
			("useCustom",     "bool",  False, _("M3 for laser (settings below)")),
			("rFeed",         "int",   "",    _("Feed rapid G0"),"Defaults from config, if blank"),       # default = min(CNC.feedmax_x, CNC.feedmax_y)
			("spinMin",       "int",   "",    _("Laser power minimum"),"Defaults from config, if blank"), # default = Utils.config.get("CNC","spindlemin")
			("spinMax",       "int",   "",    _("Laser power maximum"),"Defaults from config, if blank"), # default = Utils.config.get("CNC","spindlemax")
		]
		self.buttons.append("exe")
		#self.help = "Plugin: Driller\n"

	# Excellon Coordsconvert
	def coord2float(self, text, unitinch):
		if '.' in text: return float(text)
		if unitinch==True: return float(text)*0.0001
		# Unit mm
		if len(text)==(6 if text[0]=='-' else 5): return int(text)*0.01
		if len(text)==(7 if text[0]=='-' else 6): return int(text)*0.001

	# Convert to systemsetting
	def convunit(self, value, unitinch):
		if unitinch==CNC.inch: return value
		if unitinch==True and CNC.inch==False: return value*25.4
		if unitinch==False and CNC.inch: return value/25.4

	# Excellon Import
	def excellonimport(self, filename, app):
		fo = open(filename,"r")
		header = None
		current_tool = None
		incrementcoord = False
		unitinch = True
		data = {"tools":{}}
		targetDepth = self.fromMm("TargetDepth")
		for row in fo.readlines():
			line = row.strip()
			if len(line)!=0:
				if line[0]!=";":
					# Read header
					if line=="M48": header = True
					if header==True:
						if (line.startswith("INCH") or line.startswith("METRIC")): unitinch = line.startswith("INCH")
						if (line=="M95" or line=="%"): header = False
						if line[0]=="T":
							# Tools
							m = re.match('(T\d+)C(.+)',line)
							data["tools"][m.group(1)]={"diameter":float(m.group(2)),"holes":[]}
						if line=="ICI": incrementcoord = True
					if header==False:
						if line[0]=="T": current_tool = line
						if line[0]=="X":
							m = re.match(r'X([\d\.-]+)Y([\d\.-]+)',line)
							# Convert to system
							x = self.convunit( self.coord2float(m.group(1), unitinch), unitinch)
							y = self.convunit( self.coord2float(m.group(2), unitinch), unitinch)
							if incrementcoord==True:
								if len(data["tools"][current_tool]["holes"])==0:
									prevx = 0
									prevy = 0
								else:
									prevx = data["tools"][current_tool]["holes"][-1][0]
									prevy = data["tools"][current_tool]["holes"][-1][1]
								x = x + prevx
								y = y + prevy
							data["tools"][current_tool]["holes"].append((x,y,targetDepth))

		unittext = 'inch' if CNC.inch else 'mm'
		n = self["name"]
		if not n or n=="default": n="Driller"
		holesCounter = 0
		blocks = []
		for tool in data["tools"]:
			dia = self.convunit(data["tools"][tool]["diameter"], unitinch)
			# Duplicates shouldn't be in the list - remove unnessesary
			blockholes = [data["tools"][tool]["holes"]]
			block,holesCount = self.create_block(blockholes ,n+" ("+str(dia)+" "+unittext+")")
			holesCounter = holesCounter+holesCount
			blocks.append(block)

		self.finish_blocks(app, blocks, holesCounter)

	# Calc subsegments -----------------------------------------------------
	# TODO Move to Utils? A few plugins use this
	def calcSegmentLength(self, xyz):
		if xyz:
			A = xyz[0]
			B = xyz[1]
			seglength_x = B[0]-A[0]
			seglength_y = B[1]-A[1]
			seglength_z = B[2]-A[2]
			return math.sqrt(seglength_x**2 + seglength_y**2 + seglength_z**2)
		else:
			return 0

	# Extract all segments from commands -----------------------------------
	# TODO Move to Utils? A few plugins use this
	def extractAllSegments(self, app, selectedBlock):
		allSegments = []
		allBlocks = app.gcode.blocks

		for bid in selectedBlock:
			bidSegments = []
			block = allBlocks[bid]
			if block.name() in ("Header", "Footer"): continue
			#if not block.enable : continue
			app.gcode.initPath(bid)
			for line in block:
				try:
					cmd = app.cnc.breakLine(app.gcode.evaluate(app.cnc.compileLine(line)))
				except:
					cmd = None

				if cmd:
					app.cnc.motionStart(cmd)
					xyz = app.cnc.motionPath()
					app.cnc.motionEnd()

					if xyz:
						# Exclude if fast move or z only movement
						G0 = 'G0' in cmd[0].upper()
						Zonly = (xyz[0][0] == xyz[1][0] and xyz[0][1] == xyz[1][1])
						exclude = G0 or Zonly

						# Save length for later use
						segLenth = self.calcSegmentLength(xyz)

						if len(xyz) < 3:
							bidSegments.append([xyz[0],xyz[1],exclude,segLenth])
						else:
							for i in range(len(xyz)-1):
								bidSegments.append([xyz[i],xyz[i+1],exclude,segLenth])
			# Append bidSegments to allSegments
			allSegments.append(bidSegments)

		# Disable used block
		for bid in selectedBlock:
			block = allBlocks[bid]
			if block.name() in ("Header", "Footer"): continue
			block.enable = False

		return allSegments

	# ----------------------------------------------------------------------
	def execute(self, app):
		# Get inputs
		holesDistance = self.fromMm("HolesDistance")
		peck = self.fromMm("Peck")
		dwell = self["Dwell"]
		useAnchor = self["useAnchor"]
		excellonFileName = self["File"]

		# ------------------------------------------------------------------
		# Custom for laser in M3 mode
		self.useCustom = self["useCustom"]

		self.rFeed = int(min(CNC.feedmax_x, CNC.feedmax_y))
		if self["rFeed"]:
			self.rFeed = self["rFeed"]

		self.spinMin = Utils.config.get("CNC","spindlemin")
		if self["spinMin"]:
			self.spinMin = self["spinMin"]

		self.spinMax = Utils.config.get("CNC","spindlemax")
		if self["spinMax"]:
			self.spinMax = self["spinMax"]
		# ------------------------------------------------------------------

		# Check inputs
		if holesDistance <=0 and useAnchor == False:
			app.setStatus(_("Driller abort: Distance must be > 0"))
			return

		if peck <0:
			app.setStatus(_("Driller abort: Peck must be >= 0"))
			return

		if dwell <0:
			app.setStatus(_("Driller abort: Dwell time >= 0, here time runs only forward!"))
			return

		if excellonFileName != "":
			if os.path.isfile(excellonFileName):
				self.excellonimport(excellonFileName, app)
			else:
				app.setStatus(_("Driller abort: Excellon-File not a file"))
			return

		# Get selected blocks from editor
		selBlocks = app.editor.getSelectedBlocks()
		if not selBlocks:
			app.editor.selectAll()
			selBlocks = app.editor.getSelectedBlocks()

		if not selBlocks:
			app.setStatus(_("Driller abort: Please select some path"))
			return

		# Get all segments from gcode
		allSegments = self.extractAllSegments(app,selBlocks)

		# Create holes locations
		allHoles=[]
		for bidSegment in allSegments:
			if len(bidSegment)==0:
				continue

			if useAnchor == True:
				bidHoles = []
				for idx, anchor in enumerate(bidSegment):
					if idx > 0:
						if self.useCustom:
							if (anchor[0][0] != anchor[1][0] and anchor[0][1] != anchor[1][1]):
								newHolePoint = (anchor[1][0],anchor[1][1],anchor[0][2])
								bidHoles.append(newHolePoint)
						else:
							newHolePoint = (anchor[0][0],anchor[0][1],anchor[0][2])
							bidHoles.append(newHolePoint)
			else:
				# Sum all path length
				fullPathLength = 0.0
				for s in bidSegment:
					fullPathLength += s[3]

				# Calc rest
				holes = fullPathLength // holesDistance
				rest = fullPathLength - (holesDistance * (holes))
				# Travel along the path
				elapsedLength = rest / 2.0 # Equally distribute rest, as option???
				bidHoles = []
				while elapsedLength <= fullPathLength:
					# Search best segment to apply line interpolation
					bestSegment = bidSegment[0]
					segmentsSum = 0.0
					perc = 0.0
					for s in bidSegment:
						bestSegment = s
						segmentLength = bestSegment[3]
						perc = (elapsedLength-segmentsSum) / segmentLength
						segmentsSum += segmentLength
						if segmentsSum > elapsedLength : break

					# Fist point
					x1 = bestSegment[0][0]
					y1 = bestSegment[0][1]
					z1 = bestSegment[0][2]
					# Last point
					x2 = bestSegment[1][0]
					y2 = bestSegment[1][1]
					z2 = bestSegment[1][2]

					# Check if segment is not excluded
					if not bestSegment[2]:
						newHolePoint = (x1 + perc*(x2-x1),
							y1 + perc*(y2-y1),
							z1 + perc*(z2-z1))
						bidHoles.append(newHolePoint)

					# Go to next hole
					elapsedLength += holesDistance
			# Remove duplicates
			bidHoles = list(OrderedDict.fromkeys(bidHoles))
			# Add bidHoles to allHoles
			allHoles.append(bidHoles)

		# Write gcommands from allSegments to the drill block
		blocks = []
		n = self["name"]
		if not n or n=="default": n="Driller"
		if self.useCustom: n += "-laser-mode"
		block,holesCount = self.create_block(allHoles,n)
		blocks.append(block)
		self.finish_blocks(app, blocks, holesCount)

	# Write gcommands from allHoles to the drill block
	def create_block(self, holes, name):
		targetDepth = self.fromMm("TargetDepth")
		peck = self.fromMm("Peck")
		dwell = self["Dwell"]
		block = Block(name)
		holesCount = 0

		if self.useCustom:
			block.append("M3 S0")
		else:
			block.append(CNC.zsafe())

		for bid in holes:
			for xH,yH,zH in bid:
				holesCount += 1

				if self.useCustom:
					block.append(CNC.grapid(x=xH, y=yH) + CNC.fmt(' F', self.rFeed))
				else:
					#block.append(CNC.zsafe()) # Moved up
					block.append(CNC.grapid(xH,yH))

				if peck != 0:
					z = 0
					while z > targetDepth:
						z = max(z-peck, targetDepth)
						if self.useCustom:
							block.append("( --- WARNING! Peck is not setup for laser mode --- )")
							break
						else:
							block.append(CNC.zenter(zH + z))
							block.append(CNC.zsafe())

				if self.useCustom:
					block.append("G1 S%s" % (self.spinMax))
					block.append(CNC.gline(x=xH, y=yH))
				else:
					block.append(CNC.zenter(zH + targetDepth))

				# Dwell time only on last pass
				if dwell != 0:
					block.append(CNC.gcode(4, [("P", dwell)]))

				if self.useCustom:
					block.append("G1 S%s" % (self.spinMin))
				else:
					block.append(CNC.zsafe())

		# Gcode Zsafe on finish
		if self.useCustom:
			block.append("M5")
		else:
			block.append(CNC.zsafe())
		return (block,holesCount)

	# Insert created blocks
	def finish_blocks(self, app, blocks, numberholes):
		active = app.activeBlock()
		if active==0: active=1
		app.gcode.insBlocks(active, blocks, "Driller")
		app.refresh()
		app.setStatus(_("Generated Driller: %d holes")%numberholes)




#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	Filippo Rivato
# Date:		20 December 2015
# Porting of midi2cnc https://github.com/michthom/MIDI-to-CNC (GPLv2+)

from __future__ import absolute_import
from __future__ import print_function

__author__	= "Filippo Rivato"
__email__	= "f.rivato@gmail.com"

__name__	= "Midi2CNC"
__version__	= "0.0.1"

import math
from CNC import CNC,Block
from ToolsPage import Plugin


#==============================================================================
#Midi2CNC class
#==============================================================================
class Midi2CNC:
	def __init__(self,name="Midi2CNC"):
		self.name = name


#==============================================================================
# Create pyrograph
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Sound your machine from a midi file")

	def __init__(self, master):
		Plugin.__init__(self, master, "Midi2CNC")
		self.icon  = "midi2cnc"
		self.group = "Artistic"

		self.axes_dict = dict( {
		'X':[0],       'Y':[1],    'Z':[2],
		'XY':[0,1],    'YX':[1,0], 'XZ':[0,2],
		'ZX':[2,0],    'YZ':[1,2], 'ZY':[2,1],
		'XYZ':[0,1,2], 'XZY':[0,2,1],
		'YXZ':[1,0,2], 'YZX':[1,2,0],
		'ZXY':[2,0,1], 'ZYX':[2,1,0]
		})

		self.variables = [
			("name",	 "db" ,	   "", _("Name")),
			("ppu_X"  ,   "float" , 200.0, _("Pulse per unit for X")),
			("ppu_Y"  ,   "float" , 200.0, _("Pulse per unit for Y")),
			("ppu_Z"  ,  "float" ,  200.0, _("Pulse per unit for Z")),
			("max_X"  ,  "int" ,       50, _("Maximum X travel")),
			("max_Y"  ,  "int" ,       50, _("Maximum Y travel")),
			("max_Z"  ,  "int" ,       20, _("Maximum Z travel")),
			("AxisUsed", ",".join(self.axes_dict.keys()), "XYZ", _("Axis to be used")),
			("File"  ,   "file" ,	   "", _("Midi to process")),
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def reached_limit(self,current, distance, direction, min, max):
		# Returns true if the proposed movement will exceed the
		# safe working limits of the machine but the movement is
		# allowable in the reverse direction
		#
		# Returns false if the movement is allowable in the
		# current direction
		#
		# Aborts if the movement is not possible in either direction

		if ( ( (current + (distance * direction)) < max ) and
			 ( (current + (distance * direction)) > min ) ):
			# Movement in the current direction is within safe limits,
			return False

		elif ( ( (current + (distance * direction)) >= max ) and
			   ( (current - (distance * direction)) >  min ) ):
			# Movement in the current direction violates maximum safe
			# value, but would be safe if the direction is reversed
			return True

		elif ( ( (current + (distance * direction)) <= min ) and
			   ( (current - (distance * direction)) <  max ) ):
			# Movement in the current direction violates minimum safe
			# value, but would be safe if the direction is reversed
			return True

		else:
			# Movement in *either* direction violates the safe working
			# envelope, so abort.
			exit(2);

	# ----------------------------------------------------------------------
	def execute(self, app):
		try:
			import midiparser as midiparser
		except:
			app.setStatus(_("Error: This plugin requires midiparser.py"))
			return

		n = self["name"]
		if not n or n=="default": n="Midi2CNC"

		fileName = self["File"]

		x=0.0
		y=0.0
		z=0.0

		x_dir=1.0;
		y_dir=1.0;
		z_dir=1.0;

		# List of MIDI channels (instruments) to import.
		# Channel 10 is percussion, so better to omit it
		channels = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]

		axes = self["AxisUsed"]
		active_axes = len(axes)

		transpose = (0,0,0)
		ppu = [ 200, 200, 200 ]
		ppu[0] = self["ppu_X"]
		ppu[1] = self["ppu_X"]
		ppu[2] = self["ppu_X"]

		safemin = [ 0, 0, 0 ]
		safemax = [ 100, 100, 50 ]
		safemax[0] = self["max_X"]
		safemax[1] = self["max_Y"]
		safemax[2] = self["max_Z"]

		try:
			midi = midiparser.File(fileName)
		except:
			app.setStatus(_("Error: Sorry can't parse the Midi file."))
			return

		noteEventList=[]
		all_channels=set()

		for track in midi.tracks:
			#channels=set()
			for event in track.events:
				if event.type == midiparser.meta.SetTempo:
					tempo=event.detail.tempo

				# filter undesired instruments
				if ((event.type == midiparser.voice.NoteOn) and (event.channel in channels)):

					if event.channel not in channels:
						channels.add(event.channel)

					# NB: looks like some use "note on (vel 0)" as equivalent to note off, so check for vel=0 here and treat it as a note-off.
					if event.detail.velocity > 0:
						noteEventList.append([event.absolute, 1, event.detail.note_no, event.detail.velocity])
					else:
						noteEventList.append([event.absolute, 0, event.detail.note_no, event.detail.velocity])

				if (event.type == midiparser.voice.NoteOff) and (event.channel in channels):
					if event.channel not in channels:
						channels.add(event.channel)
					noteEventList.append([event.absolute, 0, event.detail.note_no, event.detail.velocity])

			# Finished with this track
			if len(channels) > 0:
				msg=', ' . join(['%2d' % ch for ch in sorted(channels)])
				#print 'Processed track %d, containing channels numbered: [%s ]' % (track.number, msg)
				all_channels = all_channels.union(channels)

		# List all channels encountered
		if len(all_channels) > 0:
			msg=', ' . join(['%2d' % ch for ch in sorted(all_channels)])
			#print 'The file as a whole contains channels numbered: [%s ]' % msg

		# We now have entire file's notes with abs time from all channels
		# We don't care which channel/voice is which, but we do care about having all the notes in order
		# so sort event list by abstime to dechannelify

		noteEventList.sort()
		# print noteEventList
		# print len(noteEventList)

		last_time=-0
		active_notes={} # make this a dict so we can add and remove notes by name

		# Start the output
		#Init blocks
		blocks = []
		block = Block(self.name)
		block.append("(Midi2CNC)")
		block.append("(Midi:%s)" % fileName)
		block.append(CNC.zsafe())
		block.append(CNC.grapid(0,0))
		block.append(CNC.zenter(0))

		for note in noteEventList:
			# note[timestamp, note off/note on, note_no, velocity]
			if last_time < note[0]:

				freq_xyz=[0,0,0]
				feed_xyz=[0,0,0]
				distance_xyz=[0,0,0]
				duration=0

				# "i" ranges from 0 to "the number of active notes *or* the number of active axes,
				# whichever is LOWER". Note that the range operator stops
				# short of the maximum, so this means 0 to 2 at most for a 3-axis machine.
				# E.g. only look for the first few active notes to play despite what
				# is going on in the actual score.

				for i in range(0, min(len(active_notes.values()), active_axes)):

					# Which axis are should we be writing to?
					#
					j = self.axes_dict.get(axes)[i]

					# Debug
					# print"Axes %s: item %d is %d" % (axes_dict.get(args.axes), i, j)

					# Sound higher pitched notes first by sorting by pitch then indexing by axis
					#
					nownote=sorted(active_notes.values(), reverse=True)[i]

					# MIDI note 69	 = A4(440Hz)
					# 2 to the power (69-69) / 12 * 440 = A4 440Hz
					# 2 to the power (64-69) / 12 * 440 = E4 329.627Hz
					#
					freq_xyz[j] = pow(2.0, (nownote-69 + transpose[j])/12.0)*440.0

					# Here is where we need smart per-axis feed conversions
					# to enable use of X/Y *and* Z on a Makerbot
					#
					# feed_xyz[0] = X; feed_xyz[1] = Y; feed_xyz[2] = Z;
					#
					# Feed rate is expressed in mm / minutes so 60 times
					# scaling factor is required.

					feed_xyz[j] = ( freq_xyz[j] * 60.0 ) / ppu[j]

					# Get the duration in seconds from the MIDI values in divisions, at the given tempo
					duration = ( ( ( note[0] - last_time ) + 0.0 ) / ( midi.division + 0.0 ) * ( tempo / 1000000.0 ) )

					# Get the actual relative distance travelled per axis in mm
					distance_xyz[j] = ( feed_xyz[j] * duration ) / 60.0

				# Now that axes can be addressed in any order, need to make sure
				# that all of them are silent before declaring a rest is due.
				if distance_xyz[0] + distance_xyz[1] + distance_xyz[2] > 0:
					# At least one axis is playing, so process the note into
					# movements
					combined_feedrate = math.sqrt(feed_xyz[0]**2 + feed_xyz[1]**2 + feed_xyz[2]**2)

					# Turn around BEFORE crossing the limits of the
					# safe working envelope
					if self.reached_limit( x, distance_xyz[0], x_dir, safemin[0], safemax[0] ):
						x_dir = x_dir * -1
					x = (x + (distance_xyz[0] * x_dir))

					if self.reached_limit( y, distance_xyz[1], y_dir, safemin[1], safemax[1] ):
						y_dir = y_dir * -1
					y = (y + (distance_xyz[1] * y_dir))

					if self.reached_limit( z, distance_xyz[2], z_dir, safemin[2], safemax[2] ):
						z_dir = z_dir * -1
					z = (z + (distance_xyz[2] * z_dir))

					v = (x,y,z)
					block.append(CNC.glinev(1,v,combined_feedrate))

				else:
					# Handle 'rests' in addition to notes.
					duration = (((note[0]-last_time)+0.0)/(midi.division+0.0)) * (tempo/1000000.0)
					block.append(CNC.gcode(4, [("P",duration)]))

				# finally, set this absolute time as the new starting time
				last_time = note[0]

			if note[1]==1: # Note on
				if active_notes.has_key(note[2]):
					pass
				else:
					# key and value are the same, but we don't really care.
					active_notes[note[2]]=note[2]
			elif note[1]==0: # Note off
				if(active_notes.has_key(note[2])):
					active_notes.pop(note[2])

		blocks.append(block)
		active = app.activeBlock()
		if active==0: active=1
		app.gcode.insBlocks(active, blocks, "Midi2CNC")
		app.refresh()
		app.setStatus(_("Generated Midi2CNC, ready to play?"))



#!/usr/bin/python
# -*- coding: ascii -*-
# $Id$
#
# Author:	https://github.com/carlosgs
# Date:	14-Sep-2015

from __future__ import absolute_import
from __future__ import print_function
__author__ = "Carlos Garcia Saura"
__email__  = ""
__name__   = _("Bowl")

import math
from CNC import CNC,Block
from ToolsPage import Plugin


#==============================================================================
# Bowl class
#==============================================================================
class Bowl:
	def __init__(self, name):
		self.name = name

	#----------------------------------------------------------------------
	# r       = sphere radius
	# res     = pressure angle
	# pocket  = progressive (carves the sphere pocketing each layer)
	#----------------------------------------------------------------------
	def calc(self, D, res, pocket):
		blocks = []
		block = Block(self.name)

		# Load tool and material settings
		toolDiam = CNC.vars['diameter']
		toolRadius = toolDiam/2.
		stepz  = CNC.vars['stepz']
		stepxy = toolDiam*(CNC.vars['stepover']/100.)

		if toolDiam <= 0 or stepxy <= 0 or stepz <= 0 or D <= 0 or res <= 0:
			return blocks

		currDepth = 0.

		def setCutFeedrate():
			block.append(CNC.gcode(1, [("f",CNC.vars["cutfeed"])]))

		def addCircumference(radius):
			block.append(CNC.garc(2,radius, 0., i=-radius))

		# Mills a circle, pocketing it if needed
		def addSingleCircle(radius, depth):
			if pocket:
				block.append(CNC.grapid(0., 0.))
				block.append(CNC.zenter(depth))
				setCutFeedrate()
				currRadius = 0.
				while radius > currRadius+stepxy:
					currRadius += stepxy
					block.append(CNC.gline(currRadius, 0))
					addCircumference(currRadius)
				if radius-currRadius > 0:
					block.append(CNC.gline(radius, 0))
					addCircumference(radius)
			else:
				block.append(CNC.grapid(radius, 0.))
				block.append(CNC.zenter(depth))
				setCutFeedrate()
				addCircumference(radius)

		# Mills a circle in steps of height "stepz"
		def addCircle(radius, depth, currDepth):
			while depth < currDepth-stepz:
				currDepth -= stepz
				addSingleCircle(radius, currDepth)
			if currDepth-depth > 0:
				addSingleCircle(radius, depth)
			return depth

		block.append(CNC.zsafe())
		r = D/2.
		r -= toolRadius # Removes offset of ball-end tool
		angleInc = res
		currAngle = 0.
		angle = math.pi/2. # 90 degrees
		while angle > currAngle+angleInc:
			currAngle += angleInc
			radius = r * math.cos(-currAngle)
			# Removes vertical offset (centers the ball tool in Z=0, rather than the tip)
			depth  = r * math.sin(-currAngle) - toolRadius
			currDepth = addCircle(radius, depth, currDepth)
		if angle-currAngle > 0:
			radius = r * math.cos(-angle)
			depth  = r * math.sin(-angle) - toolRadius
			currDepth = addCircle(radius, depth, currDepth)

		blocks.append(block)
		return blocks


#==============================================================================
# Create a simple Bowl
#==============================================================================
class Tool(Plugin):
	__doc__ = _("Generate a bowl cavity")

	def __init__(self, master):
		Plugin.__init__(self, master, "Bowl")
		self.group = "Generator"
		self.icon  = "bowl"
		self.variables = [
			("name",      "db",     "",  _("Name")),
			("D",         "mm",   30.0,  _("Diameter")),
			("res",    "float",   10.0,  _("Resolution (degrees)")),
			("pocket",    "bool",    1,  _("Progressive"))
		]
		self.buttons.append("exe")

	# ----------------------------------------------------------------------
	def execute(self, app):
		n = self["name"]
		if not n or n=="default": n="Bowl"
		bowl = Bowl(n)
		blocks = bowl.calc(self.fromMm("D"), math.radians(self["res"]), self["pocket"])
		if len(blocks) > 0:
			active = app.activeBlock()
			if active==0: active=1
			app.gcode.insBlocks(active, blocks, "Create BOWL")
			app.refresh()
			app.setStatus(_("Generated: BOWL"))
		else:
			app.setStatus(_("Error: Check the Bowl and End Mill parameters"))



from __future__ import absolute_import
import datetime


def camel_to_underscore(name):
    '''Convert camel case style naming to underscore style naming

    If there are existing underscores they will be collapsed with the
    to-be-added underscores. Multiple consecutive capital letters will not be
    split except for the last one.

    >>> camel_to_underscore('SpamEggsAndBacon')
    'spam_eggs_and_bacon'
    >>> camel_to_underscore('Spam_and_bacon')
    'spam_and_bacon'
    >>> camel_to_underscore('Spam_And_Bacon')
    'spam_and_bacon'
    >>> camel_to_underscore('__SpamAndBacon__')
    '__spam_and_bacon__'
    >>> camel_to_underscore('__SpamANDBacon__')
    '__spam_and_bacon__'
    '''
    output = []
    for i, c in enumerate(name):
        if i > 0:
            pc = name[i - 1]
            if c.isupper() and not pc.isupper() and pc != '_':
                # Uppercase and the previous character isn't upper/underscore?
                # Add the underscore
                output.append('_')
            elif i > 3 and not c.isupper():
                # Will return the last 3 letters to check if we are changing
                # case
                previous = name[i - 3:i]
                if previous.isalpha() and previous.isupper():
                    output.insert(len(output) - 1, '_')

        output.append(c.lower())

    return ''.join(output)


def timesince(dt, default='just now'):
    '''
    Returns string representing 'time since' e.g.
    3 days ago, 5 hours ago etc.

    >>> now = datetime.datetime.now()
    >>> timesince(now)
    'just now'
    >>> timesince(now - datetime.timedelta(seconds=1))
    '1 second ago'
    >>> timesince(now - datetime.timedelta(seconds=2))
    '2 seconds ago'
    >>> timesince(now - datetime.timedelta(seconds=60))
    '1 minute ago'
    >>> timesince(now - datetime.timedelta(seconds=61))
    '1 minute and 1 second ago'
    >>> timesince(now - datetime.timedelta(seconds=62))
    '1 minute and 2 seconds ago'
    >>> timesince(now - datetime.timedelta(seconds=120))
    '2 minutes ago'
    >>> timesince(now - datetime.timedelta(seconds=121))
    '2 minutes and 1 second ago'
    >>> timesince(now - datetime.timedelta(seconds=122))
    '2 minutes and 2 seconds ago'
    >>> timesince(now - datetime.timedelta(seconds=3599))
    '59 minutes and 59 seconds ago'
    >>> timesince(now - datetime.timedelta(seconds=3600))
    '1 hour ago'
    >>> timesince(now - datetime.timedelta(seconds=3601))
    '1 hour and 1 second ago'
    >>> timesince(now - datetime.timedelta(seconds=3602))
    '1 hour and 2 seconds ago'
    >>> timesince(now - datetime.timedelta(seconds=3660))
    '1 hour and 1 minute ago'
    >>> timesince(now - datetime.timedelta(seconds=3661))
    '1 hour and 1 minute ago'
    >>> timesince(now - datetime.timedelta(seconds=3720))
    '1 hour and 2 minutes ago'
    >>> timesince(now - datetime.timedelta(seconds=3721))
    '1 hour and 2 minutes ago'
    >>> timesince(datetime.timedelta(seconds=3721))
    '1 hour and 2 minutes ago'
    '''
    if isinstance(dt, datetime.timedelta):
        diff = dt
    else:
        now = datetime.datetime.now()
        diff = abs(now - dt)

    periods = (
        (diff.days / 365, 'year', 'years'),
        (diff.days % 365 / 30, 'month', 'months'),
        (diff.days % 30 / 7, 'week', 'weeks'),
        (diff.days % 7, 'day', 'days'),
        (diff.seconds / 3600, 'hour', 'hours'),
        (diff.seconds % 3600 / 60, 'minute', 'minutes'),
        (diff.seconds % 60, 'second', 'seconds'),
    )

    output = []
    for period, singular, plural in periods:
        if int(period):
            if int(period) == 1:
                output.append('%d %s' % (period, singular))
            else:
                output.append('%d %s' % (period, plural))

    if output:
        return '%s ago' % ' and '.join(output[:2])

    return default



# -*- coding: ascii -*-
# $Id$
#
# Author: vvlachoudis@gmail.com
# Date: 24-Aug-2014

from __future__ import absolute_import
from __future__ import print_function
__author__ = "Vasilis Vlachoudis"
__email__  = "Vasilis.Vlachoudis@cern.ch"

import os
import re
import sys
#import cgi
import json
import urllib
import tempfile
import threading

from CNC import CNC
from Utils import prgpath

try:
	import urlparse
except ImportError:
	import urllib.parse as urlparse

try:
	from PIL import Image
except ImportError:
	Image = None

try:
	import BaseHTTPServer as HTTPServer
except ImportError:
	import http.server as HTTPServer

import Camera

HOSTNAME = "localhost"
port     = 8080

httpd    = None
webpath  = "%s/pendant"%(prgpath)
iconpath = "%s/icons/"%(prgpath)


#==============================================================================
# Simple Pendant controller for CNC
#==============================================================================
class Pendant(HTTPServer.BaseHTTPRequestHandler):
	camera = None

	#----------------------------------------------------------------------
	def log_message(self, fmt, *args):
		# Only requests to the main page log them, all other ignore
		if args[0].startswith("GET / ") or args[0].startswith("GET /send"):
			args = list(args)
			args[0] = self.address_string()+'" : "'+args[0]
			HTTPServer.BaseHTTPRequestHandler.log_message(self, fmt, *args)

	#----------------------------------------------------------------------
	def do_HEAD(self, rc=200, content="text/html", cl=0):
		self.send_response(rc)
		self.send_header("Content-type", content)
		if cl != 0:
			self.send_header("Content-length", cl)
		self.end_headers()

	#----------------------------------------------------------------------
	def do_GET(self):
		"""Respond to a GET request."""
		if "?" in self.path:
			page,arg = self.path.split("?",1)
			arg = dict(urlparse.parse_qsl(arg))
		else:
			page = self.path
			arg = None

#		print self.path,type(self.path)
#		print page
#		print arg

		if page == "/send":
			if arg is None: return
			for key,value in arg.items():
				if key=="gcode":
					for line in value.split('\n'):
						httpd.app.queue.put(line+"\n")
				elif key=="cmd":
					httpd.app.pendant.put(urllib.unquote(value))
			#send empty response so browser does not generate errors
			self.do_HEAD(200, "text/text", cl=len(""))
			self.wfile.write("")

		elif page == "/state":
			tmp = {}
			for name in ["controller", "state", "pins", "color", "msg", "wx", "wy", "wz", "G", "OvFeed", "OvRapid", "OvSpindle"]:
				tmp[name] = CNC.vars[name]
			contentToSend = json.dumps(tmp)
			self.do_HEAD(200, content="text/text", cl=len(contentToSend))
			self.wfile.write(contentToSend)

		elif page == "/config":
			snd = {}
			snd["rpmmax"] = httpd.app.get("CNC","spindlemax")
			contentToSend = json.dumps(snd)
			self.do_HEAD(200, content="text/text", cl=len(contentToSend))
			self.wfile.write(contentToSend)

		elif page == "/icon":
			if arg is None: return
			filename = os.path.join(iconpath, arg["name"]+".gif")
			self.do_HEAD(200, content="image/gif", cl=os.path.getsize(filename))
			try:
				f = open(filename,"rb")
				self.wfile.write(f.read())
				f.close()
			except:
				pass

		elif page == "/canvas":
			if not Image: return
			with tempfile.NamedTemporaryFile(suffix='.ps') as tmp:
				httpd.app.canvas.postscript(
					file=tmp.name,
					colormode='color',
				)
				tmp.flush()
				try:
					with tempfile.NamedTemporaryFile(suffix='.gif') as out:
						Image.open(tmp.name).save(out.name, 'GIF')
						out.flush()
						out.seek(0)
						self.do_HEAD(200, content="image/gif", cl=os.path.getsize(tmp.name))
						self.wfile.write(out.read())
				except:
					filename = os.path.join(iconpath, "warn.gif")
					self.do_HEAD(200, content="image/gif", cl=os.path.getsize(filename))
					try:
						f = open(filename,"rb")
						self.wfile.write(f.read())
						f.close()
					except:
						pass

		elif page == "/camera":
			if not Camera.hasOpenCV(): return
			if Pendant.camera is None:
				Pendant.camera = Camera.Camera("webcam")
				Pendant.camera.start()

			if Pendant.camera.read():
				Pendant.camera.save("camera.jpg")
				#cv.imwrite("camera.jpg",img)
				self.do_HEAD(200, content="image/jpeg", cl=os.path.getsize("camera.jpg"))
				try:
					f = open("camera.jpg","rb")
					self.wfile.write(f.read())
					f.close()
				except:
					pass
		else:
			self.mainPage(page[1:])

	#----------------------------------------------------------------------
	def deal_post_data(self):
		boundary = self.headers.plisttext.split("=")[1]
		remainbytes = int(self.headers['content-length'])
		line = self.rfile.readline()
		remainbytes -= len(line)
		if not boundary in line:
			return (False, "Content NOT begin with boundary")
		line = self.rfile.readline()
		remainbytes -= len(line)
		fn = re.findall(r'Content-Disposition.*name="file"; filename="(.*)"', line)
		if not fn:
			return (False, "Can't find out file name...")
		path = os.path.expanduser("~")
		path = os.path.join(path, "bCNCUploads")
		if not os.path.exists(path):
		    os.makedirs(path)
		fn = os.path.join(path, fn[0])
		line = self.rfile.readline()
		remainbytes -= len(line)
		line = self.rfile.readline()
		remainbytes -= len(line)
		try:
			out = open(fn, 'wb')
		except IOError:
			return (False, "Can't create file to write, do you have permission to write?")

		preline = self.rfile.readline()
		remainbytes -= len(preline)
		while remainbytes > 0:
			line = self.rfile.readline()
			remainbytes -= len(line)
			if boundary in line:
				preline = preline[0:-1]
				if preline.endswith('\r'):
					preline = preline[0:-1]
				out.write(preline)
				out.close()
				return (True, "%s" % fn)
			else:
				out.write(preline)
				preline = line
		return (False, "Unexpected Ends of data.")

	#----------------------------------------------------------------------
	def do_POST(self):
		result,fMsg=self.deal_post_data()
		if(result):
			httpd.app._pendantFileUploaded=fMsg
		#send empty response so browser does not generate errors
		self.do_HEAD(200, "text/text")

	# ---------------------------------------------------------------------
	def mainPage(self, page):
		global webpath

		#handle certain filetypes
		filetype = page.rpartition('.')[2]
		if   filetype == "css": self.do_HEAD(content="text/css")
		elif filetype == "js":  self.do_HEAD(content="application/x-javascript")
		elif filetype == "json": self.do_HEAD(content="application/json")
		elif filetype == "jpg" or filetype == "jpeg" : self.do_HEAD(content="image/jpeg")
		elif filetype == "gif": self.do_HEAD(content="image/gif")
		elif filetype == "png": self.do_HEAD(content="image/png")
		elif filetype == "ico": self.do_HEAD(content="image/x-icon")
		else: self.do_HEAD()

		if page == "": page = "index.html"
		try:
			f = open(os.path.join(webpath,page),"r")
			self.wfile.write(f.read())
			f.close()
		except IOError:
			self.wfile.write("""<!DOCTYPE html>
<html>
<head>
<title>Errortitle</title>
<meta name="viewport" content="width=device-width,initial-scale=1, user-scalable=yes" />
</head>
<body>
Page not found.
</body>
</html>
""")


# -----------------------------------------------------------------------------
def _server(app):
	global httpd
	server_class = HTTPServer.HTTPServer
	try:
		httpd = server_class(('', port), Pendant)
		httpd.app = app
		httpd.serve_forever()
	except:
		httpd = None


# -----------------------------------------------------------------------------
def start(app):
	global httpd

	if httpd is not None: return False
	thread = threading.Thread(target=_server, args=(app,))
	thread.start()
	return True


# -----------------------------------------------------------------------------
def stop():
	global httpd
	if httpd is None: return False
	httpd.shutdown()
	httpd = None
	if Pendant.camera: Pendant.camera.stop()
	return True

if __name__ == '__main__':
	start()
